using System;

namespace SimpleEvents
{
    
    //delegates declaration 
    //the End event will be of EndEventHandler type
    public delegate void EndEventHandler();

    //class for generating some random numbers
    class NumberGenerator
    {
        public event EndEventHandler End;   //event, we have finnished generating

        private int maximum;    //max value of generated numbers
        private Random rndm;    //pseudo-random numbers generator

        public NumberGenerator(int max)
        {
            rndm = new Random();    //initializes the random number generator
            maximum = max;
        }

        //start generating numbers for "rounds" rounds
        public void Start(int rounds)
        {
            int number;
            for (int i = 1; i <= rounds; i++)
            {
                number = rndm.Next(maximum) + 1;    //generate next random number 1 - maximum
                Console.WriteLine("{0}. round: {1}", i, number);
            }
            End();                                  //generating finnished
        }
    }


    //main program 
    class Program
    {
        //this method match the EndEventHandler delegate signature
        public static void Finnished()
        {
            Console.WriteLine("That's all...");
        }

        static void Main(string[] args)
        {
            //a new generator with maximum of 100
            NumberGenerator ng = new NumberGenerator(100);
            //registration of the Finnished method to the End event
            ng.End += new EndEventHandler(Finnished);
            //begin generating 20 numbers
            ng.Start(20);
        }
    }
}
